
/*
 * $Id: ZipfPopMdl.cc,v 1.1.2.5 1999/02/22 02:51:03 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "Rnd.h"
#include "Oid.h"
#include "TmlModel.h"
#include "ZipfPopMdl.h"

ZipfPopMdl::ZipfPopMdl(): theDistr(0) {
	theDistr = new ZipfDistr(new RndGen, 0); // world size is unknown
}

ZipfPopMdl::~ZipfPopMdl() {
	delete theDistr;
}

int ZipfPopMdl::repeatId(int lastId) {
	// old id, zipf distributed
	return theDistr->ltrial(1, lastId);
}

#if UNUSED_CODE
int ZipfPopMdl::repeatOid(const OidSelection &sel, bool cachable) {
	// XXX: ideally we should build a table of p[i]=1/k[i]^a probabilities
	// and select oid randomly based on p[i]s
	// however doing that is very expensive!

#ifdef SELECT_SMALLEST
	// so we just select the smallest oid...
	int best = 0;
	for (int i = sel.beg; i <= sel.end; ++i) {
		const int oid = sel.oids->at(i);
		if (CachableOid(oid) == cachable && (!best || best > oid))
			best = oid;
	}
#else
	// so we generate a Zipf variable and find closest oid
	const int marker = Id2Oid(theDistr->ltrial(1, sel.lastId), cachable);
	int best = 0;
	int best_dist = 0;
	for (int i = sel.beg; i <= sel.end; ++i) {
		const int oid = sel.oids->at(i);
		const int dist = abs(oid - marker);
		//cout << here << setw(5) << i << ' ' << setw(7) << oid << ' ' << setw(7) << marker << ' ' << setw(7) << best << endl;
		if (CachableOid(oid) == cachable &&
			(!best || best_dist > dist)) {
			best = oid;
			best_dist = dist;
		}
	}
#endif

	// repeat any if failed
	if (!best)
		return marker;

	return best;
}
#endif /* UNUSED_CODE */

ostream &ZipfPopMdl::print(ostream &os) const {
	return os << "zipf()";
}
