#ifndef CONNECTION_CLASS_H
#define CONNECTION_CLASS_H

/*
 * $Id: Connection.h,v 1.1.1.3 1999/02/07 05:08:46 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "NetAddr.h"
#include "Socket.h"
#include "Select.h"
#include "IOBuf.h"

class Connection {
	public:
		class HalfPipe {
			public:
				HalfPipe(Connection &aConn, IOBuf &aBuf, IODir aDir):
					theConn(aConn), theBuf(aBuf), theDir(aDir) { reset(); }

				void reset();
				void start(Select::User *u);
				void stop(Select::User *u);

			public:
				Connection &theConn;
				IOBuf &theBuf;
				IODir theDir;
				SelectReserv theReserv;
				bool isReady;
		};
		friend class HalfPipe;

	public:
		Connection();
		~Connection() { if (theSock) close(); }

		void reset();

		Socket &sock() { return theSock; }
		const NetAddr &addr() const { return theAddr; }
		bool exhausted() const { return isAtEof && !theRdBuf.contSize(); }
		bool atEof() const { return isAtEof; }
		bool bad() const { return isBad; }

		bool connect(const NetAddr &addr);
		bool accept(Socket &s, bool &fatal);
		Size read();
		Size write();
		bool close();

		void bad(bool be) { isBad = be; }

	public:
		RdBuf theRdBuf;
		WrBuf theWrBuf;

		HalfPipe theRd; // these refer to bufs; must go after them!
		HalfPipe theWr;

	protected:
		Socket theSock;
		NetAddr theAddr;
		//ConnMgr *theMgr;
		int theLocPort;

		Time theOpenTime;

		bool isBad;
		bool isAtEof;
};

#endif
