
/*
 * $Id: Clock.cc,v 1.1.1.2 1999/01/30 17:37:31 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include "Array.h"
#include "Clock.h"

static void RegisterClock(Clock *clock);
static void UnregisterClock(Clock *clock);

Clock TheClock;
static Array<Clock*> *TheClocks = 0;

/* Clock */

Clock::Clock() {
	RegisterClock(this);
}

Clock::~Clock() {
	UnregisterClock(this);
}


Time Clock::Update(bool advanceAll) {
	const Time curTime = Time::Now();
	const Time delay = curTime - TheClock;
	TheClock.update(curTime);
	if (advanceAll) {
		for (int i = TheClocks->count()-1; i >=0 ; --i) {
			Clock *&c = TheClocks->item(i);
			if (c && c != &TheClock)
				TheClocks->item(i)->update(curTime);
		}
	}
	return delay;
}


static
void RegisterClock(Clock *clock) {
	Assert(clock);
	TheClocks->append(clock);
}

static
void UnregisterClock(Clock *clock) {
	for (int i = TheClocks->count()-1; i >=0 ; --i) {
		if ((*TheClocks)[i] == clock) {
			(*TheClocks)[i] = (*TheClocks)[TheClocks->count()-1];
			TheClocks->pop(1);
			return;
		}
	}
	Assert(0); // clock not found
}

/* ClockLibCounter; internal stuff, ignore */

int ClockLibCounter::theUseCount = 0;

ClockLibCounter::ClockLibCounter() {
	if (!theUseCount++)
		TheClocks = new Array<Clock*>;
}

ClockLibCounter::~ClockLibCounter() {
	if (!--theUseCount) {
		Assert(TheClocks);
		Assert(!TheClocks->count());
		delete TheClocks;
		TheClocks = 0;
	}
}
